@@ -35,6 +35,7 @@ import (
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/clientcmd"
+	"k8s.io/client-go/tools/record"
 	"k8s.io/client-go/util/retry"
 	"k8s.io/client-go/util/workqueue"
 
@@ -45,10 +46,12 @@ import (
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/bundle"
 	olmerrors "github.com/operator-framework/operator-lifecycle-manager/pkg/controller/errors"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/operators/catalog/subscription"
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/grpc"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/reconciler"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/resolver"
 	controllerclient "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/controller-runtime/client"
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/event"
 	index "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/index"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorlister"
@@ -92,9 +95,10 @@ type Operator struct {
 	ipQueueSet               *queueinformer.ResourceQueueSet
 	nsResolveQueue           workqueue.RateLimitingInterface
 	namespace                string
+	recorder                 record.EventRecorder
 	sources                  *grpc.SourceStore
 	sourcesLastUpdate        sharedtime.SharedTime
-	resolver                 resolver.Resolver
+	resolver                 resolver.StepResolver
 	reconciler               reconciler.RegistryReconcilerFactory
 	csvProvidedAPIsIndexer   map[string]cache.Indexer
 	catalogSubscriberIndexer map[string]cache.Indexer
@@ -135,6 +139,12 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 	// Create an OperatorLister
 	lister := operatorlister.NewLister()
 
+	// eventRecorder can emit events
+	eventRecorder, err := event.NewRecorder(opClient.KubernetesInterface().CoreV1().Events(metav1.NamespaceAll))
+	if err != nil {
+		return nil, err
+	}
+
 	ssaClient, err := controllerclient.NewForConfig(config, scheme, RegistryFieldManager)
 	if err != nil {
 		return nil, err
@@ -150,7 +160,7 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 		client:                   crClient,
 		lister:                   lister,
 		namespace:                operatorNamespace,
-		resolver:                 resolver.NewOperatorsV1alpha1Resolver(lister, crClient, opClient.KubernetesInterface()),
+		recorder:                 eventRecorder,
 		catsrcQueueSet:           queueinformer.NewEmptyResourceQueueSet(),
 		subQueueSet:              queueinformer.NewEmptyResourceQueueSet(),
 		ipQueueSet:               queueinformer.NewEmptyResourceQueueSet(),
@@ -161,6 +171,8 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 	}
 	op.sources = grpc.NewSourceStore(logger, 10*time.Second, 10*time.Minute, op.syncSourceState)
 	op.reconciler = reconciler.NewRegistryReconcilerFactory(lister, opClient, configmapRegistryImage, op.now, ssaClient)
+	res := resolver.NewOperatorStepResolver(lister, crClient, opClient.KubernetesInterface(), operatorNamespace, op.sources, logger)
+	op.resolver = resolver.NewInstrumentedResolver(res, metrics.RegisterDependencyResolutionSuccess, metrics.RegisterDependencyResolutionFailure)
 
 	// Wire OLM CR sharedIndexInformers
 	crInformerFactory := externalversions.NewSharedInformerFactoryWithOptions(op.client, resyncPeriod())
@@ -187,7 +199,7 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 	op.ipQueueSet.Set(metav1.NamespaceAll, ipQueue)
 	ipQueueInformer, err := queueinformer.NewQueueInformer(
 		ctx,
-		queueinformer.WithMetricsProvider(metrics.NewMetricsInstallPlan(op.client)),
+		queueinformer.WithMetricsProvider(metrics.NewMetricsInstallPlan(op.lister.OperatorsV1alpha1().InstallPlanLister())),
 		queueinformer.WithLogger(op.logger),
 		queueinformer.WithQueue(ipQueue),
 		queueinformer.WithInformer(ipInformer.Informer()),
@@ -207,7 +219,7 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 	op.catsrcQueueSet.Set(metav1.NamespaceAll, catsrcQueue)
 	catsrcQueueInformer, err := queueinformer.NewQueueInformer(
 		ctx,
-		queueinformer.WithMetricsProvider(metrics.NewMetricsCatalogSource(op.client)),
+		queueinformer.WithMetricsProvider(metrics.NewMetricsCatalogSource(op.lister.OperatorsV1alpha1().CatalogSourceLister())),
 		queueinformer.WithLogger(op.logger),
 		queueinformer.WithQueue(catsrcQueue),
 		queueinformer.WithInformer(catsrcInformer.Informer()),
@@ -249,7 +261,7 @@ func NewOperator(ctx context.Context, kubeconfigPath string, clock utilclock.Clo
 	}
 	subQueueInformer, err := queueinformer.NewQueueInformer(
 		ctx,
-		queueinformer.WithMetricsProvider(metrics.NewMetricsSubscription(op.client)),
+		queueinformer.WithMetricsProvider(metrics.NewMetricsSubscription(op.lister.OperatorsV1alpha1().SubscriptionLister())),
 		queueinformer.WithLogger(op.logger),
 		queueinformer.WithQueue(subQueue),
 		queueinformer.WithInformer(subInformer.Informer()),
@@ -384,6 +396,7 @@ func (o *Operator) syncSourceState(state grpc.SourceState) {
 
 	switch state.State {
 	case connectivity.Ready:
+		o.resolver.Expire(state.Key)
 		if o.namespace == state.Key.Namespace {
 			namespaces, err := index.CatalogSubscriberNamespaces(o.catalogSubscriberIndexer,
 				state.Key.Name, state.Key.Namespace)
@@ -396,10 +409,9 @@ func (o *Operator) syncSourceState(state grpc.SourceState) {
 		}
 
 		o.nsResolveQueue.Add(state.Key.Namespace)
-	default:
-		if err := o.catsrcQueueSet.Requeue(state.Key.Namespace, state.Key.Name); err != nil {
-			o.logger.WithError(err).Info("couldn't requeue catalogsource from catalog status change")
-		}
+	}
+	if err := o.catsrcQueueSet.Requeue(state.Key.Namespace, state.Key.Name); err != nil {
+		o.logger.WithError(err).Info("couldn't requeue catalogsource from catalog status change")
 	}
 }
 
@@ -493,7 +505,7 @@ func (o *Operator) handleCatSrcDeletion(obj interface{}) {
 			}
 		}
 	}
-	sourceKey := resolver.CatalogKey{Name: catsrc.GetName(), Namespace: catsrc.GetNamespace()}
+	sourceKey := registry.CatalogKey{Name: catsrc.GetName(), Namespace: catsrc.GetNamespace()}
 	if err := o.sources.Remove(sourceKey); err != nil {
 		o.logger.WithError(err).Warn("error closing client")
 	}
@@ -580,7 +592,7 @@ func (o *Operator) syncConfigMap(logger *logrus.Entry, in *v1alpha1.CatalogSourc
 func (o *Operator) syncRegistryServer(logger *logrus.Entry, in *v1alpha1.CatalogSource) (out *v1alpha1.CatalogSource, continueSync bool, syncError error) {
 	out = in.DeepCopy()
 
-	sourceKey := resolver.CatalogKey{Name: in.GetName(), Namespace: in.GetNamespace()}
+	sourceKey := registry.CatalogKey{Name: in.GetName(), Namespace: in.GetNamespace()}
 	srcReconciler := o.reconciler.ReconcilerForSource(in)
 	if srcReconciler == nil {
 		// TODO: Add failure status on catalogsource and remove from sources
@@ -628,7 +640,7 @@ func (o *Operator) syncRegistryServer(logger *logrus.Entry, in *v1alpha1.Catalog
 	// requeue the catalog sync based on the polling interval, for accurate syncs of catalogs with polling enabled
 	if out.Spec.UpdateStrategy != nil {
 		logger.Debugf("requeuing registry server sync based on polling interval %s", out.Spec.UpdateStrategy.Interval.Duration.String())
-		resyncPeriod := reconciler.SyncRegistryUpdateInterval(out, time.Now())
+		resyncPeriod := reconciler.SyncRegistryUpdateInterval(out)
 		o.catsrcQueueSet.RequeueAfter(out.GetNamespace(), out.GetName(), queueinformer.ResyncWithJitter(resyncPeriod, 0.1)())
 		return
 	}
@@ -643,7 +655,7 @@ func (o *Operator) syncRegistryServer(logger *logrus.Entry, in *v1alpha1.Catalog
 func (o *Operator) syncConnection(logger *logrus.Entry, in *v1alpha1.CatalogSource) (out *v1alpha1.CatalogSource, continueSync bool, syncError error) {
 	out = in.DeepCopy()
 
-	sourceKey := resolver.CatalogKey{Name: in.GetName(), Namespace: in.GetNamespace()}
+	sourceKey := registry.CatalogKey{Name: in.GetName(), Namespace: in.GetNamespace()}
 	// update operator's view of sources
 	now := o.now()
 	address := in.Address()
@@ -837,6 +849,7 @@ func (o *Operator) syncResolvingNamespace(obj interface{}) error {
 		// ensure the installplan reference is correct
 		sub, changedIP, err := o.ensureSubscriptionInstallPlanState(logger, sub)
 		if err != nil {
+			logger.Debugf("error ensuring installplan state: %v", err)
 			return err
 		}
 		subscriptionUpdated = subscriptionUpdated || changedIP
@@ -844,6 +857,7 @@ func (o *Operator) syncResolvingNamespace(obj interface{}) error {
 		// record the current state of the desired corresponding CSV in the status. no-op if we don't know the csv yet.
 		sub, changedCSV, err := o.ensureSubscriptionCSVState(logger, sub, querier)
 		if err != nil {
+			logger.Debugf("error recording current state of CSV in status: %v", err)
 			return err
 		}
 
@@ -869,6 +883,7 @@ func (o *Operator) syncResolvingNamespace(obj interface{}) error {
 	// resolve a set of steps to apply to a cluster, a set of subscriptions to create/update, and any errors
 	steps, bundleLookups, updatedSubs, err := o.resolver.ResolveSteps(namespace, querier)
 	if err != nil {
+		go o.recorder.Event(ns, corev1.EventTypeWarning, "ResolutionFailed", err.Error())
 		return err
 	}
 
@@ -905,6 +920,8 @@ func (o *Operator) syncResolvingNamespace(obj interface{}) error {
 			logger.WithError(err).Debug("error ensuring subscription installplan state")
 			return err
 		}
+	} else {
+		logger.Debugf("no subscriptions were updated")
 	}
 
 	return nil
@@ -991,7 +1008,7 @@ func (o *Operator) ensureSubscriptionCSVState(logger *logrus.Entry, sub *v1alpha
 		if err := querier.Queryable(); err != nil {
 			return nil, false, err
 		}
-		b, _, _ := querier.FindReplacement(&csv.Spec.Version.Version, sub.Status.CurrentCSV, sub.Spec.Package, sub.Spec.Channel, resolver.CatalogKey{Name: sub.Spec.CatalogSource, Namespace: sub.Spec.CatalogSourceNamespace})
+		b, _, _ := querier.FindReplacement(&csv.Spec.Version.Version, sub.Status.CurrentCSV, sub.Spec.Package, sub.Spec.Channel, registry.CatalogKey{Name: sub.Spec.CatalogSource, Namespace: sub.Spec.CatalogSourceNamespace})
 		if b != nil {
 			o.logger.Tracef("replacement %s bundle found for current bundle %s", b.CsvName, sub.Status.CurrentCSV)
 			out.Status.State = v1alpha1.SubscriptionStateUpgradeAvailable
@@ -1152,6 +1169,7 @@ type UnpackedBundleReference struct {
 	CatalogSourceName      string `json:"catalogSourceName"`
 	CatalogSourceNamespace string `json:"catalogSourceNamespace"`
 	Replaces               string `json:"replaces"`
+	Properties             string `json:"properties"`
 }
 
 // unpackBundles makes one walk through the bundlelookups and attempts to progress them
@@ -1197,6 +1215,7 @@ func (o *Operator) unpackBundles(plan *v1alpha1.InstallPlan) (bool, *v1alpha1.In
 				CatalogSourceName:      res.CatalogSourceRef.Name,
 				CatalogSourceNamespace: res.CatalogSourceRef.Namespace,
 				Replaces:               res.Replaces,
+				Properties:             res.Properties,
 			}
 			r, err := json.Marshal(&ref)
 			if err != nil {
@@ -1559,16 +1578,7 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 
 	ensurer := newStepEnsurer(kubeclient, crclient, dynamicClient)
 	r := newManifestResolver(plan.GetNamespace(), o.lister.CoreV1().ConfigMapLister(), o.logger)
-
-	// CRDs should be installed via the default OLM (cluster-admin) client and not the scoped client specified by the AttenuatedServiceAccount
-	// the StepBuilder is currently only implemented for CRD types
-	// TODO give the StepBuilder both OLM and scoped clients when it supports new scoped types
-	builderKubeClient, _, builderDynamicClient, err := o.clientAttenuator.AttenuateClientWithServiceAccount(nil)
-	if err != nil {
-		o.logger.Errorf("failed to get a client for plan execution- %v", err)
-		return err
-	}
-	b := newBuilder(builderKubeClient, builderDynamicClient, o.csvProvidedAPIsIndexer, r, o.logger)
+	b := newBuilder(kubeclient, dynamicClient, o.csvProvidedAPIsIndexer, r, o.logger)
 
 	for i, step := range plan.Status.Plan {
 		doStep := true
@@ -2062,16 +2072,23 @@ func (o *Operator) apiresourceFromGVK(gvk schema.GroupVersionKind) (metav1.APIRe
 }
 
 const (
-	PrometheusRuleKind = "PrometheusRule"
-	ServiceMonitorKind = "ServiceMonitor"
+	PrometheusRuleKind        = "PrometheusRule"
+	ServiceMonitorKind        = "ServiceMonitor"
+	PodDisruptionBudgetKind   = "PodDisruptionBudget"
+	PriorityClassKind         = "PriorityClass"
+	VerticalPodAutoscalerKind = "VerticalPodAutoscaler"
 )
 
+var supportedKinds = map[string]struct{}{
+	PrometheusRuleKind:        {},
+	ServiceMonitorKind:        {},
+	PodDisruptionBudgetKind:   {},
+	PriorityClassKind:         {},
+	VerticalPodAutoscalerKind: {},
+}
+
 // isSupported returns true if OLM supports this type of CustomResource.
 func isSupported(kind string) bool {
-	switch kind {
-	case PrometheusRuleKind, ServiceMonitorKind:
-		return true
-	default:
-		return false
-	}
+	_, ok := supportedKinds[kind]
+	return ok
 }
