@@ -7,7 +7,6 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/blang/semver"
 	"github.com/operator-framework/operator-registry/pkg/api"
 	opregistry "github.com/operator-framework/operator-registry/pkg/registry"
 	"github.com/stretchr/testify/require"
@@ -33,61 +32,6 @@ func RequireStepsEqual(t *testing.T, expectedSteps, steps []*v1alpha1.Step) {
 	}
 }
 
-func NewGenerationFromOperators(ops ...OperatorSurface) *NamespaceGeneration {
-	g := NewEmptyGeneration()
-
-	for _, op := range ops {
-		if err := g.AddOperator(op); err != nil {
-			fmt.Printf("error adding operator: %s\n", err.Error())
-			return nil
-		}
-	}
-	return g
-}
-
-func NewFakeOperatorSurface(name, pkg, channel, replaces, src, startingCSV string, providedCRDs, requiredCRDs, providedAPIServices, requiredAPIServices []opregistry.APIKey) *Operator {
-	providedAPISet := EmptyAPISet()
-	requiredAPISet := EmptyAPISet()
-	providedCRDAPISet := EmptyAPISet()
-	requiredCRDAPISet := EmptyAPISet()
-	providedAPIServiceAPISet := EmptyAPISet()
-	requiredAPIServiceAPISet := EmptyAPISet()
-	version := semver.MustParse("0.0.0")
-
-	for _, p := range providedCRDs {
-		providedCRDAPISet[p] = struct{}{}
-		providedAPISet[p] = struct{}{}
-	}
-	for _, r := range requiredCRDs {
-		requiredCRDAPISet[r] = struct{}{}
-		requiredAPISet[r] = struct{}{}
-	}
-	for _, p := range providedAPIServices {
-		providedAPIServiceAPISet[p] = struct{}{}
-		providedAPISet[p] = struct{}{}
-	}
-	for _, r := range requiredAPIServices {
-		requiredAPIServiceAPISet[r] = struct{}{}
-		requiredAPISet[r] = struct{}{}
-	}
-	b := bundle(name, pkg, channel, replaces, providedCRDAPISet, requiredCRDAPISet, providedAPIServiceAPISet, requiredAPIServiceAPISet)
-
-	return &Operator{
-		providedAPIs: providedAPISet,
-		requiredAPIs: requiredAPISet,
-		name:         name,
-		replaces:     replaces,
-		version:      &version,
-		sourceInfo: &OperatorSourceInfo{
-			Package:     pkg,
-			Channel:     channel,
-			StartingCSV: startingCSV,
-			Catalog:     CatalogKey{src, src + "-namespace"},
-		},
-		bundle: b,
-	}
-}
-
 func csv(name, replaces string, ownedCRDs, requiredCRDs, ownedAPIServices, requiredAPIServices APISet, permissions, clusterPermissions []v1alpha1.StrategyDeploymentPermissions) *v1alpha1.ClusterServiceVersion {
 	var singleInstance = int32(1)
 	strategy := v1alpha1.StrategyDetailsDeployment{
@@ -211,7 +155,7 @@ func u(object runtime.Object) *unstructured.Unstructured {
 	return &unstructured.Unstructured{Object: unst}
 }
 
-func apiSetToGVk(crds, apis APISet) (out []*api.GroupVersionKind) {
+func apiSetToGVK(crds, apis APISet) (out []*api.GroupVersionKind) {
 	out = make([]*api.GroupVersionKind, 0)
 	for a := range crds {
 		out = append(out, &api.GroupVersionKind{
@@ -232,7 +176,112 @@ func apiSetToGVk(crds, apis APISet) (out []*api.GroupVersionKind) {
 	return
 }
 
-func bundle(name, pkg, channel, replaces string, providedCRDs, requiredCRDs, providedAPIServices, requiredAPIServices APISet) *api.Bundle {
+func apiSetToDependencies(crds, apis APISet) (out []*api.Dependency) {
+	if len(crds)+len(apis) == 0 {
+		return nil
+	}
+	out = make([]*api.Dependency, 0)
+	for a := range crds {
+		val, err := json.Marshal(opregistry.GVKDependency{
+			Group:   a.Group,
+			Kind:    a.Kind,
+			Version: a.Version,
+		})
+		if err != nil {
+			panic(err)
+		}
+		out = append(out, &api.Dependency{
+			Type:  opregistry.GVKType,
+			Value: string(val),
+		})
+	}
+	for a := range apis {
+		val, err := json.Marshal(opregistry.GVKDependency{
+			Group:   a.Group,
+			Kind:    a.Kind,
+			Version: a.Version,
+		})
+		if err != nil {
+			panic(err)
+		}
+		out = append(out, &api.Dependency{
+			Type:  opregistry.GVKType,
+			Value: string(val),
+		})
+	}
+	if len(out) == 0 {
+		return nil
+	}
+	return
+}
+
+func apiSetToProperties(crds, apis APISet, deprecated bool) (out []*api.Property) {
+	out = make([]*api.Property, 0)
+	for a := range crds {
+		val, err := json.Marshal(opregistry.GVKProperty{
+			Group:   a.Group,
+			Kind:    a.Kind,
+			Version: a.Version,
+		})
+		if err != nil {
+			panic(err)
+		}
+		out = append(out, &api.Property{
+			Type:  opregistry.GVKType,
+			Value: string(val),
+		})
+	}
+	for a := range apis {
+		val, err := json.Marshal(opregistry.GVKProperty{
+			Group:   a.Group,
+			Kind:    a.Kind,
+			Version: a.Version,
+		})
+		if err != nil {
+			panic(err)
+		}
+		out = append(out, &api.Property{
+			Type:  opregistry.GVKType,
+			Value: string(val),
+		})
+	}
+	if deprecated {
+		val, err := json.Marshal(opregistry.DeprecatedProperty{})
+		if err != nil {
+			panic(err)
+		}
+		out = append(out, &api.Property{
+			Type:  opregistry.DeprecatedType,
+			Value: string(val),
+		})
+	}
+	if len(out) == 0 {
+		return nil
+	}
+	return
+}
+
+type bundleOpt func(*api.Bundle)
+
+func withSkipRange(skipRange string) bundleOpt {
+	return func(b *api.Bundle) {
+		b.SkipRange = skipRange
+	}
+}
+
+func withSkips(skips []string) bundleOpt {
+	return func(b *api.Bundle) {
+		b.Skips = skips
+	}
+}
+
+func withVersion(version string) bundleOpt {
+	return func(b *api.Bundle) {
+		b.Version = version
+	}
+}
+
+func bundle(name, pkg, channel, replaces string, providedCRDs, requiredCRDs, providedAPIServices, requiredAPIServices APISet, opts ...bundleOpt) *api.Bundle {
 	csvJson, err := json.Marshal(csv(name, replaces, providedCRDs, requiredCRDs, providedAPIServices, requiredAPIServices, nil, nil))
 	if err != nil {
 		panic(err)
@@ -247,16 +296,23 @@ func bundle(name, pkg, channel, replaces string, providedCRDs, requiredCRDs, pro
 		objs = append(objs, string(crdJson))
 	}
 
-	return &api.Bundle{
+	b := &api.Bundle{
 		CsvName:      name,
 		PackageName:  pkg,
 		ChannelName:  channel,
 		CsvJson:      string(csvJson),
 		Object:       objs,
 		Version:      "0.0.0",
-		ProvidedApis: apiSetToGVk(providedCRDs, providedAPIServices),
-		RequiredApis: apiSetToGVk(requiredCRDs, requiredAPIServices),
+		ProvidedApis: apiSetToGVK(providedCRDs, providedAPIServices),
+		RequiredApis: apiSetToGVK(requiredCRDs, requiredAPIServices),
+		Replaces:     replaces,
+		Dependencies: apiSetToDependencies(requiredCRDs, requiredAPIServices),
+		Properties:   apiSetToProperties(providedCRDs, providedAPIServices, false),
 	}
+	for _, f := range opts {
+		f(b)
+	}
+	return b
 }
 
 func stripManifests(bundle *api.Bundle) *api.Bundle {
@@ -300,8 +356,8 @@ func bundleWithPermissions(name, pkg, channel, replaces string, providedCRDs, re
 		ChannelName:  channel,
 		CsvJson:      string(csvJson),
 		Object:       objs,
-		ProvidedApis: apiSetToGVk(providedCRDs, providedAPIServices),
-		RequiredApis: apiSetToGVk(requiredCRDs, requiredAPIServices),
+		ProvidedApis: apiSetToGVK(providedCRDs, providedAPIServices),
+		RequiredApis: apiSetToGVK(requiredCRDs, requiredAPIServices),
 	}
 }
 
@@ -311,8 +367,8 @@ func withReplaces(operator *Operator, replaces string) *Operator {
 }
 
 // NewFakeSourceQuerier builds a querier that talks to fake registry stubs for testing
-func NewFakeSourceQuerier(bundlesByCatalog map[CatalogKey][]*api.Bundle) *NamespaceSourceQuerier {
-	sources := map[CatalogKey]registry.ClientInterface{}
+func NewFakeSourceQuerier(bundlesByCatalog map[registry.CatalogKey][]*api.Bundle) *NamespaceSourceQuerier {
+	sources := map[registry.CatalogKey]registry.ClientInterface{}
 	for catKey, bundles := range bundlesByCatalog {
 		source := &fakes.FakeClientInterface{}
 		source.GetBundleThatProvidesStub = func(ctx context.Context, groupOrName, version, kind string) (*api.Bundle, error) {
@@ -347,12 +403,7 @@ func NewFakeSourceQuerier(bundlesByCatalog map[CatalogKey][]*api.Bundle) *Namesp
 
 		source.GetReplacementBundleInPackageChannelStub = func(ctx context.Context, bundleName, packageName, channelName string) (*api.Bundle, error) {
 			for _, b := range bundles {
-				csv, err := V1alpha1CSVFromBundle(b)
-				if err != nil {
-					panic(err)
-				}
-				replaces := csv.Spec.Replaces
-				if replaces == bundleName && b.ChannelName == channelName && b.PackageName == packageName {
+				if b.Replaces == bundleName && b.ChannelName == channelName && b.PackageName == packageName {
 					return b, nil
 				}
 			}
@@ -443,23 +494,3 @@ func getPkgName(pkgChan string) string {
 	s := strings.Split(pkgChan, "/")
 	return s[0]
 }
-
-// NewFakeSourceQuerier builds a querier that talks to fake registry stubs for testing
-func NewFakeSourceQuerierCustomReplacement(catKey CatalogKey, bundle *api.Bundle) *NamespaceSourceQuerier {
-	sources := map[CatalogKey]registry.ClientInterface{}
-	source := &fakes.FakeClientInterface{}
-	source.GetBundleThatProvidesStub = func(ctx context.Context, groupOrName, version, kind string) (*api.Bundle, error) {
-		return nil, fmt.Errorf("no bundle found")
-	}
-	source.GetBundleInPackageChannelStub = func(ctx context.Context, packageName, channelName string) (*api.Bundle, error) {
-		return nil, fmt.Errorf("no bundle found")
-	}
-	source.GetBundleStub = func(ctx context.Context, packageName, channelName, csvName string) (*api.Bundle, error) {
-		return nil, fmt.Errorf("no bundle found")
-	}
-	source.GetReplacementBundleInPackageChannelStub = func(ctx context.Context, bundleName, packageName, channelName string) (*api.Bundle, error) {
-		return bundle, nil
-	}
-	sources[catKey] = source
-	return NewNamespaceSourceQuerier(sources)
-}
